@@ -5,7 +5,14 @@
 #include <drm/drm_framebuffer.h>
 #include <drm/drm_modeset_helper.h>
 
-MALLOC_DECLARE(DRM_MEM_KMS);
+struct drm_gem_object *
+drm_gem_fb_get_obj(struct drm_framebuffer *fb, unsigned int plane)
+{
+	if (plane >= DRM_FORMAT_MAX_PLANES)
+		return NULL;
+
+	return fb->obj[plane];
+}
 
 void
 drm_gem_fb_destroy(struct drm_framebuffer *fb)
