@@ -366,41 +366,6 @@ out:
 	return i915_error_to_vmf_fault(err);
 }
 
-static void set_address_limits(struct vm_area_struct *area,
-			       struct i915_vma *vma,
-			       unsigned long obj_offset,
-			       unsigned long *start_vaddr,
-			       unsigned long *end_vaddr)
-{
-	unsigned long vm_start, vm_end, vma_size; /* user's memory parameters */
-	long start, end; /* memory boundaries */
-
-	/*
-	 * Let's move into the ">> PAGE_SHIFT"
-	 * domain to be sure not to lose bits
-	 */
-	vm_start = area->vm_start >> PAGE_SHIFT;
-	vm_end = area->vm_end >> PAGE_SHIFT;
-	vma_size = vma->size >> PAGE_SHIFT;
-
-	/*
-	 * Calculate the memory boundaries by considering the offset
-	 * provided by the user during memory mapping and the offset
-	 * provided for the partial mapping.
-	 */
-	start = vm_start;
-	start -= obj_offset;
-	start += vma->gtt_view.partial.offset;
-	end = start + vma_size;
-
-	start = max_t(long, start, vm_start);
-	end = min_t(long, end, vm_end);
-
-	/* Let's move back into the "<< PAGE_SHIFT" domain */
-	*start_vaddr = (unsigned long)start << PAGE_SHIFT;
-	*end_vaddr = (unsigned long)end << PAGE_SHIFT;
-}
-
 static vm_fault_t vm_fault_gtt(struct vm_fault *vmf)
 {
 #define MIN_CHUNK_PAGES (SZ_1M >> PAGE_SHIFT)
@@ -413,18 +378,14 @@ static vm_fault_t vm_fault_gtt(struct vm_fault *vmf)
 	struct i915_ggtt *ggtt = to_gt(i915)->ggtt;
 	bool write = area->vm_flags & VM_WRITE;
 	struct i915_gem_ww_ctx ww;
-	unsigned long obj_offset;
-	unsigned long start, end; /* memory boundaries */
 	intel_wakeref_t wakeref;
 	struct i915_vma *vma;
 	pgoff_t page_offset;
-	unsigned long pfn;
 	int srcu;
 	int ret;
 
-	obj_offset = area->vm_pgoff - drm_vma_node_start(&mmo->vma_node);
+	/* We don't use vmf->pgoff since that has the fake offset */
 	page_offset = (vmf->address - area->vm_start) >> PAGE_SHIFT;
-	page_offset += obj_offset;
 
 	trace_i915_gem_object_fault(obj, page_offset, true, write);
 
@@ -446,7 +407,7 @@ retry:
 	if (ret)
 		goto err_rpm;
 
-	ret = intel_gt_reset_trylock(ggtt->vm.gt, &srcu);
+	ret = intel_gt_reset_lock_interruptible(ggtt->vm.gt, &srcu);
 	if (ret)
 		goto err_pages;
 
@@ -499,7 +460,16 @@ retry:
 	}
 
 	/* Access to snoopable pages through the GTT is incoherent. */
-	if (obj->cache_level != I915_CACHE_NONE && !HAS_LLC(i915)) {
+	/*
+	 * For objects created by userspace through GEM_CREATE with pat_index
+	 * set by set_pat extension, coherency is managed by userspace, make
+	 * sure we don't fail handling the vm fault by calling
+	 * i915_gem_object_has_cache_level() which always return true for such
+	 * objects. Otherwise this helper function would fall back to checking
+	 * whether the object is un-cached.
+	 */
+	if (!(i915_gem_object_has_cache_level(obj, I915_CACHE_NONE) ||
+	      HAS_LLC(i915))) {
 		ret = -EFAULT;
 		goto err_unpin;
 	}
@@ -508,14 +478,12 @@ retry:
 	if (ret)
 		goto err_unpin;
 
-	set_address_limits(area, vma, obj_offset, &start, &end);
-
-	pfn = (ggtt->gmadr.start + i915_ggtt_offset(vma)) >> PAGE_SHIFT;
-	pfn += (start - area->vm_start) >> PAGE_SHIFT;
-	pfn += obj_offset - vma->gtt_view.partial.offset;
-
 	/* Finally, remap it using the new GTT offset */
-	ret = remap_io_mapping(area, start, pfn, end - start, &ggtt->iomap);
+	ret = remap_io_mapping(area,
+			       area->vm_start + (vma->gtt_view.partial.offset << PAGE_SHIFT),
+			       (ggtt->gmadr.start + i915_ggtt_offset(vma)) >> PAGE_SHIFT,
+			       min_t(u64, vma->size, area->vm_end - area->vm_start),
+			       &ggtt->iomap);
 	if (ret)
 		goto err_fence;
 
@@ -1065,7 +1033,7 @@ i915_gem_object_mmap(struct drm_i915_gem_object *obj,
 			i915_gem_object_put(obj);
 			return -EINVAL;
 		}
-		vma->vm_flags &= ~VM_MAYWRITE;
+		vm_flags_clear(vma, VM_MAYWRITE);
 	}
 
 	anon = mmap_singleton(to_i915(dev));
@@ -1074,7 +1042,7 @@ i915_gem_object_mmap(struct drm_i915_gem_object *obj,
 		return PTR_ERR(anon);
 	}
 
-	vma->vm_flags |= VM_PFNMAP | VM_DONTEXPAND | VM_DONTDUMP | VM_IO;
+	vm_flags_set(vma, VM_PFNMAP | VM_DONTEXPAND | VM_DONTDUMP | VM_IO);
 
 	/*
 	 * We keep the ref on mmo->obj, not vm_file, but we require
@@ -1202,8 +1170,6 @@ int i915_gem_fb_mmap(struct drm_i915_gem_object *obj, struct vm_area_struct *vma
 		mmo = mmap_offset_attach(obj, mmap_type, NULL);
 		if (IS_ERR(mmo))
 			return PTR_ERR(mmo);
-
-		vma->vm_pgoff += drm_vma_node_start(&mmo->vma_node);
 	}
 
 	/*
