@@ -125,8 +125,7 @@ drm_vprintf(struct drm_printer *p, const char *fmt, va_list *va)
 
 /**
  * struct drm_print_iterator - local struct used with drm_printer_coredump
- * @data: Pointer to the devcoredump output buffer, can be NULL if using
- * drm_printer_coredump to determine size of devcoredump
+ * @data: Pointer to the devcoredump output buffer
  * @start: The offset within the buffer to start writing
  * @remain: The number of bytes to write for this iteration
  */
@@ -171,57 +170,6 @@ struct drm_print_iterator {
  *			coredump_read, ...)
  *	}
  *
- * The above example has a time complexity of O(N^2), where N is the size of the
- * devcoredump. This is acceptable for small devcoredumps but scales poorly for
- * larger ones.
- *
- * Another use case for drm_coredump_printer is to capture the devcoredump into
- * a saved buffer before the dev_coredump() callback. This involves two passes:
- * one to determine the size of the devcoredump and another to print it to a
- * buffer. Then, in dev_coredump(), copy from the saved buffer into the
- * devcoredump read buffer.
- *
- * For example::
- *
- *	char *devcoredump_saved_buffer;
- *
- *	ssize_t __coredump_print(char *buffer, ssize_t count, ...)
- *	{
- *		struct drm_print_iterator iter;
- *		struct drm_printer p;
- *
- *		iter.data = buffer;
- *		iter.start = 0;
- *		iter.remain = count;
- *
- *		p = drm_coredump_printer(&iter);
- *
- *		drm_printf(p, "foo=%d\n", foo);
- *		...
- *		return count - iter.remain;
- *	}
- *
- *	void coredump_print(...)
- *	{
- *		ssize_t count;
- *
- *		count = __coredump_print(NULL, INT_MAX, ...);
- *		devcoredump_saved_buffer = kvmalloc(count, GFP_KERNEL);
- *		__coredump_print(devcoredump_saved_buffer, count, ...);
- *	}
- *
- *	void coredump_read(char *buffer, loff_t offset, size_t count,
- *			   void *data, size_t datalen)
- *	{
- *		...
- *		memcpy(buffer, devcoredump_saved_buffer + offset, count);
- *		...
- *	}
- *
- * The above example has a time complexity of O(N*2), where N is the size of the
- * devcoredump. This scales better than the previous example for larger
- * devcoredumps.
- *
  * RETURNS:
  * The &drm_printer object
  */
@@ -768,9 +716,6 @@ void __drm_err(const char *function_name, const char *format, ...);
 #define drm_dbg_kms_ratelimited(drm, fmt, ...) \
 	__DRM_DEFINE_DBG_RATELIMITED(KMS, drm, fmt, ## __VA_ARGS__)
 
-/* NOTE: this is deprecated in favor of drm_dbg_kms_ratelimited(NULL, ...). */
-#define DRM_DEBUG_KMS_RATELIMITED(fmt, ...) drm_dbg_kms_ratelimited(NULL, fmt, ## __VA_ARGS__)
-
 /*
  * struct drm_device based WARNs
  *
